@@ -1,15 +1,14 @@
 mod named_file;
 pub use named_file::*;
 
-use bytes::BytesMut;
-use futures::Stream;
+use std::cmp;
 use std::future::Future;
+use std::io::{self, Read, Seek};
 use std::pin::Pin;
 use std::task::{Context, Poll};
-use std::{
-    cmp,
-    io::{self, Read, Seek},
-};
+
+use bytes::BytesMut;
+use futures_util::{ready, Stream};
 
 pub(crate) enum ChunkedState<T> {
     File(Option<T>),
@@ -57,7 +56,7 @@ where
                 self.poll_next(cx)
             }
             ChunkedState::Future(ref mut fut) => {
-                let (file, buf) = futures::ready!(Pin::new(fut).poll(cx))
+                let (file, buf) = ready!(Pin::new(fut).poll(cx))
                     .map_err(|_| io::Error::new(io::ErrorKind::Other, "BlockingErr"))??;
                 self.state = ChunkedState::File(Some(file));
 
@@ -72,9 +71,15 @@ where
 
 #[cfg(test)]
 mod test {
-    use super::{ChunkedState, FileChunk};
-    use futures::stream::StreamExt;
     use std::io::Cursor;
+    use std::path::Path;
+    use std::str::FromStr;
+
+    use futures_util::stream::StreamExt;
+    use mime::Mime;
+
+    use super::*;
+    use crate::http::header::HeaderValue;
 
     #[tokio::test]
     async fn test_chunk_read() {
@@ -97,4 +102,23 @@ mod test {
 
         assert_eq!(mock.into_inner(), result)
     }
+    #[tokio::test]
+    async fn test_named_file_builder() {
+        let src = "../examples/static/test/test1.txt";
+        // println!("current path: {:?}", std::env::current_dir());
+        // println!("current current_exe: {:?}", std::env::current_exe());
+        let file = NamedFile::builder(src.into())
+            .with_attached_filename("attach.file")
+            .with_buffer_size(8888)
+            .with_content_type(Mime::from_str("text/html").unwrap())
+            .build()
+            .await
+            .unwrap();
+        assert_eq!(file.path(), Path::new(src));
+        assert_eq!(file.content_type(), &Mime::from_str("text/html").unwrap());
+        assert_eq!(
+            file.content_disposition(),
+            &HeaderValue::from_static("attachment; filename=attach.file")
+        );
+    }
 }
