@@ -475,7 +475,7 @@ impl Request {
     #[inline]
     pub async fn read_text(&mut self) -> Result<&str, ReadError> {
         match self.payload().await {
-            Ok(body) => Ok(std::str::from_utf8(&body)?),
+            Ok(body) => Ok(std::str::from_utf8(body)?),
             Err(_) => Err(ReadError::General("read text from body failed".into())),
         }
     }
@@ -544,3 +544,40 @@ pub(crate) async fn read_body_bytes(body: Body) -> Result<Vec<u8>, ReadError> {
         .map_err(|_| ReadError::General("read body bytes error".into()))
         .map(|d| d.to_vec())
 }
+
+#[cfg(test)]
+mod tests {
+    use serde::Deserialize;
+
+    use super::*;
+
+    #[tokio::test]
+    async fn test_read_text() {
+        let mut req = Request::from_hyper(
+            hyper::Request::builder()
+                .uri("http://127.0.0.1:7878/hello")
+                .header("content-type", "text/plain")
+                .body("hello".into())
+                .unwrap(),
+        );
+        assert_eq!(req.read_from_text::<String>().await.unwrap(), "hello");
+    }
+    #[tokio::test]
+    async fn test_read_json() {
+        #[derive(Deserialize, Eq, PartialEq, Debug)]
+        struct User {
+            name: String,
+        }
+        let mut req = Request::from_hyper(
+            hyper::Request::builder()
+                .uri("http://127.0.0.1:7878/hello")
+                .header("content-type", "application/json")
+                .body(r#"{"name": "jobs"}"#.into())
+                .unwrap(),
+        );
+        assert_eq!(
+            req.read_from_json::<User>().await.unwrap(),
+            User { name: "jobs".into() }
+        );
+    }
+}
