@@ -5,18 +5,22 @@ use std::io::{self, Write};
 use std::pin::Pin;
 use std::task::{self, Poll};
 
+use async_compression::tokio::bufread::{BrotliDecoder, DeflateDecoder, GzipDecoder};
 use bytes::{BufMut, Bytes, BytesMut};
 use cookie::{Cookie, CookieJar};
-use futures::{Stream, TryStreamExt};
+use encoding_rs::{Encoding, UTF_8};
+use futures_util::{Stream, StreamExt, TryStreamExt};
 use http::version::Version;
-use hyper::Method;
+use mime::Mime;
+use serde::de::DeserializeOwned;
 use serde::Serialize;
+use tokio::io::{AsyncReadExt, BufReader};
 
 pub use http::response::Parts;
 
 use super::errors::*;
-use super::header::{self, HeaderMap, HeaderValue, InvalidHeaderValue, SET_COOKIE};
-use crate::http::{Request, StatusCode};
+use super::header::{self, HeaderMap, HeaderValue, InvalidHeaderValue, CONTENT_ENCODING, SET_COOKIE};
+use crate::http::StatusCode;
 
 #[allow(clippy::type_complexity)]
 pub enum Body {
@@ -24,6 +28,71 @@ pub enum Body {
     Bytes(BytesMut),
     Stream(Pin<Box<dyn Stream<Item = Result<Bytes, Box<dyn StdError + Send + Sync>>> + Send>>),
 }
+impl Body {
+    pub fn is_empty(&self) -> bool {
+        if let Body::Empty = *self {
+            true
+        } else {
+            false
+        }
+    }
+    pub async fn take_json<T: DeserializeOwned>(&mut self) -> crate::Result<T> {
+        let full = self.take_bytes().await?;
+        serde_json::from_slice(&full).map_err(crate::Error::new)
+    }
+    pub async fn take_text(&mut self, charset: &str, compress: Option<&str>) -> crate::Result<String> {
+        let charset = Encoding::for_label(charset.as_bytes()).unwrap_or(UTF_8);
+        let mut full = self.take_bytes().await?;
+        if let Some(algo) = compress {
+            match algo {
+                "gzip" => {
+                    let mut reader = GzipDecoder::new(BufReader::new(full.as_ref()));
+                    let mut buf = vec![];
+                    reader.read_to_end(&mut buf).await.map_err(crate::Error::new)?;
+                    full = Bytes::from(buf);
+                }
+                "deflate" => {
+                    let mut reader = DeflateDecoder::new(BufReader::new(full.as_ref()));
+                    let mut buf = vec![];
+                    reader.read_to_end(&mut buf).await.map_err(crate::Error::new)?;
+                    full = Bytes::from(buf);
+                }
+                "br" => {
+                    let mut reader = BrotliDecoder::new(BufReader::new(full.as_ref()));
+                    let mut buf = vec![];
+                    reader.read_to_end(&mut buf).await.map_err(crate::Error::new)?;
+                    full = Bytes::from(buf);
+                }
+                _ => {
+                    tracing::error!(compress = %algo, "unknown compress format");
+                }
+            }
+        }
+        let (text, _, _) = charset.decode(&full);
+        if let Cow::Owned(s) = text {
+            return Ok(s);
+        }
+        unsafe {
+            // decoding returned Cow::Borrowed, meaning these bytes
+            // are already valid utf8
+            Ok(String::from_utf8_unchecked(full.to_vec()))
+        }
+    }
+    pub async fn take_bytes(&mut self) -> crate::Result<Bytes> {
+        let bytes = match self {
+            Self::Empty => Bytes::new(),
+            Self::Bytes(bytes) => std::mem::take(bytes).freeze(),
+            Self::Stream(stream) => {
+                let mut bytes = BytesMut::new();
+                while let Some(chunk) = stream.next().await {
+                    bytes.extend(chunk.map_err(crate::Error::new)?);
+                }
+                bytes.freeze()
+            }
+        };
+        Ok(bytes)
+    }
+}
 
 impl Stream for Body {
     type Item = Result<Bytes, Box<dyn StdError + Send + Sync>>;
@@ -83,7 +152,7 @@ impl Response {
     /// Create a request from an hyper::Request.
     ///
     /// This constructor consumes the hyper::Request.
-    pub fn from_hyper(req: hyper::Response<hyper::Body>) -> Response {
+    pub fn from_hyper(res: hyper::Response<hyper::Body>) -> Response {
         let (
             http::response::Parts {
                 status,
@@ -93,7 +162,7 @@ impl Response {
                 ..
             },
             body,
-        ) = req.into_parts();
+        ) = res.into_parts();
 
         // Set the request cookies, if they exist.
         let cookies = if let Some(header) = headers.get("Cookie") {
@@ -159,20 +228,55 @@ impl Response {
     pub fn take_body(&mut self) -> Option<Body> {
         self.body.take()
     }
+    pub async fn take_json<T: DeserializeOwned>(&mut self) -> crate::Result<T> {
+        match &mut self.body {
+            Some(body) => body.take_json().await,
+            None => Err(crate::Error::new("body is none")),
+        }
+    }
+    pub async fn take_text(&mut self) -> crate::Result<String> {
+        self.take_text_with_charset("utf-8").await
+    }
+    pub async fn take_text_with_charset(&mut self, default_charset: &str) -> crate::Result<String> {
+        match &mut self.body {
+            Some(body) => {
+                let content_type = self
+                    .headers
+                    .get(header::CONTENT_TYPE)
+                    .and_then(|value| value.to_str().ok())
+                    .and_then(|value| value.parse::<Mime>().ok());
+                let charset = content_type
+                    .as_ref()
+                    .and_then(|mime| mime.get_param("charset").map(|charset| charset.as_str()))
+                    .unwrap_or(default_charset);
+                body.take_text(
+                    charset,
+                    self.headers.get(CONTENT_ENCODING).and_then(|v| v.to_str().ok()),
+                )
+                .await
+            }
+            None => Err(crate::Error::new("body is none")),
+        }
+    }
+    pub async fn take_bytes(&mut self) -> crate::Result<Bytes> {
+        match &mut self.body {
+            Some(body) => body.take_bytes().await,
+            None => Err(crate::Error::new("body is none")),
+        }
+    }
 
     // `write_back` is used to put all the data added to `self`
     // back onto an `hyper::Response` so that it is sent back to the
     // client.
     //
     // `write_back` consumes the `Response`.
-    pub(crate) async fn write_back(self, req: &mut Request, res: &mut hyper::Response<hyper::Body>) {
+    pub(crate) async fn write_back(self, res: &mut hyper::Response<hyper::Body>) {
         *res.headers_mut() = self.headers;
 
         // Default to a 404 if no response code was set
         *res.status_mut() = self.status_code.unwrap_or(StatusCode::NOT_FOUND);
 
-        if let Method::HEAD = *req.method() {
-        } else if let Some(body) = self.body {
+        if let Some(body) = self.body {
             match body {
                 Body::Bytes(bytes) => {
                     *res.body_mut() = hyper::Body::from(Bytes::from(bytes));
@@ -226,7 +330,7 @@ impl Response {
         self.cookies.remove(Cookie::named(name));
     }
     #[inline]
-    pub fn status_code(&mut self) -> Option<StatusCode> {
+    pub fn status_code(&self) -> Option<StatusCode> {
         self.status_code
     }
 
@@ -243,12 +347,17 @@ impl Response {
     //     self.headers.get_one("Content-Type").and_then(|v| v.parse().ok())
     // }
 
+    #[inline]
+    pub fn http_error(&self) -> Option<&HttpError> {
+        self.http_error.as_ref()
+    }
     #[inline]
     pub fn set_http_error(&mut self, err: HttpError) {
         self.status_code = Some(err.code);
         self.http_error = Some(err);
         self.commit();
     }
+
     #[inline]
     pub fn render_json<T: Serialize>(&mut self, data: &T) {
         let mut cache = Cache::with_capacity(128);
@@ -473,9 +582,17 @@ impl Write for Cache {
 mod test {
     use super::Body;
     use bytes::BytesMut;
-    use futures::stream::{iter, StreamExt};
+    use futures_util::stream::{iter, StreamExt};
     use std::error::Error;
 
+    #[test]
+    fn test_body_empty() {
+        let body = Body::Bytes(BytesMut::from("hello"));
+        assert!(!body.is_empty());
+        let body = Body::Empty;
+        assert!(body.is_empty());
+    }
+
     #[tokio::test]
     async fn test_body_stream1() {
         let mut body = Body::Bytes(BytesMut::from("hello"));
